/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.checkers;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.PathMatcher;

import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.entities.SAREFExample;
import fr.emse.gitlab.saref.entities.SAREFRepository;
import fr.emse.gitlab.saref.managers.RepositoryManager;

/**
 * Checks TS 103 673 Clause 9.6: Ontology examples
 * 
 */
public class Clause_9_6_Checker extends AbstractClauseChecker {
	
	private static enum MESSAGE {
		missing, ioexception;
	}

	public Clause_9_6_Checker(RepositoryManager repositoryManager) {
		super(repositoryManager, Clause_9_6_Checker.class);
	}

	@Override
	public void check() throws SAREFPipelineException {
		File dir = new File(repository.getDirectory(), "examples");
		if (!dir.isDirectory()) {
			return;
		}
		
		new Clause_9_6_1_Checker(repositoryManager).check();
		
		for(SAREFExample example: repository.getExamples().values()) {
			new Clause_9_6_2_Checker(repositoryManager, example).check();
			new Clause_9_6_3_Checker(repositoryManager, example).check();
		}

//		try {
//			check_9_6_1(repository);
//		} catch (IOException ex) {
//			log(getMessage(MESSAGE.IOEXCEPTION), Mode.DEVELOP, Mode.RELEASE, Mode.PORTAL);
//		}
	}
	
	public void check_9_6_1(SAREFRepository repository) throws SAREFPipelineException {
//		The examples directory of the SAREF project version directory should contain at least one file.
//		Files contained in the examples directory are called the example documents of the SAREF project version. 
//		Their goal is to illustrate how the ontology can be used in practice
//		Every example document shall have the extension .ttl. It shall contain the sources of a consistent OWL 2 DL ontology [TODO add REF], in the Turtle 1.1 [TODO add REF] format.
//		Main classes and properties defined by the SAREF project version ontology should be illustrated with at least one example document.
//		Named individuals defined by the SAREF project version ontology may be illustrated with some example document.
//		The following clauses specify every example document, taking as example an example document example12.ttl.
//
//		boolean notTurtle = Files.walk(dir.toPath()).anyMatch(p -> {
//			return !ttlMatcher.matches(p);
//		});
//		if (!notTurtle) {
//			log(getMessage(MESSAGE., null), null);
//			errorLogger
//					.warn("The `examples` folder should contain at least one example file (some `.ttl` document)");
		
		
		
//		File examplesDir = new File(version.getRepository().getDirectory(), "examples");
//		try {
//			for (Iterator<Path> it = Files.walk(examplesDir.toPath()).filter(p -> {
//				return ttlMatcher.matches(p);
//			}).iterator();it.hasNext();) {
//				Path p = it.next();
//				Path rel = examplesDir.toPath().relativize(p);
//				String iri = version.getResource().getURI() + "example/"
//						+ rel.toString().substring(0, rel.toString().length() - 4);
//				String exampleTestSuiteName = version.toString() + " testing example " + rel;
//				testSuiteNames.put(iri, exampleTestSuiteName);
//				File exampleFile = p.toFile();
//				try {
//					new ReadExamples(exampleTestSuiteName).doJob(dataset, version, iri, exampleFile);
//				} catch (SAREFPipelineException ex) {
//					LOG.error(version.toString() + " Found errors for example  " + rel
//							+ ". This example will be ignored.");
//				}
//			}
//		} catch (Exception ex) {
//			LOG.error(version.toString() + " Error while walking through the examples. They will be ignored.", ex);
//			continue;
//		}

	}
}
