/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.managers.parsers;

import org.apache.jena.query.Dataset;
import org.apache.jena.query.ReadWrite;
import org.apache.jena.rdf.model.Model;
import org.semanticweb.owlapi.formats.RDFDocumentFormat;
import org.semanticweb.owlapi.formats.TurtleDocumentFormatFactory;
import org.semanticweb.owlapi.io.OWLOntologyDocumentSource;
import org.semanticweb.owlapi.io.OWLParserException;
import org.semanticweb.owlapi.model.OWLDocumentFormat;
import org.semanticweb.owlapi.model.OWLDocumentFormatFactory;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyLoaderConfiguration;

import fr.emse.gitlab.saref.managers.DatasetManager;

/**
 *
 * @author Maxime Lefrançois
 *
 */
public class JenaModelOntologyParser extends AbstractJenaModelOntologyParser {

	private final DatasetManager datasetManager;

	public JenaModelOntologyParser(DatasetManager datasetManager) {
		this.datasetManager = datasetManager;
	}

	@Override
	public OWLDocumentFormatFactory getSupportedFormat() {
		return new TurtleDocumentFormatFactory();
	}

	@Override
	public OWLDocumentFormat parse(OWLOntologyDocumentSource source, OWLOntology ontology,
			OWLOntologyLoaderConfiguration config) {
		
		Dataset dataset = datasetManager.getDataset();

		String iri = source.getDocumentIRI().toString();

		boolean ownTransaction = false;
		if (dataset.transactionMode() == ReadWrite.WRITE) {
			throw new OWLParserException("already in writing transation");
		}
		if (dataset.transactionMode() == null) {
			ownTransaction = true;
			dataset.begin(ReadWrite.READ);
		}
		Model model = datasetManager.getModel(iri);
		if (model == null) {
			throw new OWLParserException("no graph named " + iri + " in the dataset.");
		}
		
		RDFDocumentFormat format = new JenaModelFormat();
		parse(source, ontology, config, model, format);

		if (ownTransaction) {
			dataset.end();
		}

		return format;
	}

}
